﻿using Microsoft.Win32;
using System;
using System.IO;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace Notatnik
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private string ścieżkaPliku;

        private bool _czyTekstZmieniony = false;
        private bool czyTekstZmieniony
        {
            get
            {
                return _czyTekstZmieniony;
            }
            set
            {
                if (!_czyTekstZmieniony && value) etykietaPaskaStanu.Text += "*";
                if (_czyTekstZmieniony && !value) etykietaPaskaStanu.Text = etykietaPaskaStanu.Text.Replace('*', ' ');
                _czyTekstZmieniony = value;                
            }
        }

        private bool czyWczytanyPlik
        {
            get
            {
                return !string.IsNullOrWhiteSpace(ścieżkaPliku);
            }
        }

        private OpenFileDialog openFileDialog;
        private SaveFileDialog saveFileDialog;

        public MainWindow()
        {
            InitializeComponent();

            openFileDialog = new OpenFileDialog();
            openFileDialog.Title = "Wybierz plik tekstowy";
            openFileDialog.DefaultExt = "txt";
            openFileDialog.Filter = "Pliki tekstowe (*.txt)|*.txt|Pliki XML (*.xml, *.xaml)|*.xml;*.xaml|Pliki C# (*.cs)|*.cs|Wszystkie pliki (*.*)|*.*";
            openFileDialog.FilterIndex = 1;

            saveFileDialog = new SaveFileDialog();
            saveFileDialog.Title = "Zapisz plik tekstowy";
            saveFileDialog.DefaultExt = "txt";
            saveFileDialog.Filter = openFileDialog.Filter;
            saveFileDialog.FilterIndex = 1;
            saveFileDialog.Tag = false;
        }

        private void openMenuItem_Click(object sender, RoutedEventArgs e)
        {
            if(czyWczytanyPlik)
            {
                openFileDialog.InitialDirectory = Path.GetDirectoryName(ścieżkaPliku);
                openFileDialog.FileName = Path.GetFileName(ścieżkaPliku);
            }

            bool? wynik = openFileDialog.ShowDialog();
            if(wynik.HasValue && wynik.Value)
            {
                ścieżkaPliku = openFileDialog.FileName;
                textBox.Text = File.ReadAllText(ścieżkaPliku, Encoding.UTF8);
                czyTekstZmieniony = false;
                etykietaPaskaStanu.Text = Path.GetFileName(ścieżkaPliku);
            }
        }

        private void zapisz()
        {
            File.WriteAllText(ścieżkaPliku, textBox.Text, Encoding.UTF8);
            czyTekstZmieniony = false;
        }

        private void saveAsMenuItem_Click(object sender, RoutedEventArgs e)
        {
            if (czyWczytanyPlik)
            {
                saveFileDialog.InitialDirectory = Path.GetDirectoryName(ścieżkaPliku);
                saveFileDialog.FileName = Path.GetFileName(ścieżkaPliku);
            }

            bool? wynik = saveFileDialog.ShowDialog();
            saveFileDialog.Tag = false;
            if (wynik.HasValue && wynik.Value)
            {
                ścieżkaPliku = saveFileDialog.FileName;
                zapisz();
                etykietaPaskaStanu.Text = Path.GetFileName(ścieżkaPliku);
                saveFileDialog.Tag = true;
            }
        }

        private void saveMenuItem_Click(object sender, RoutedEventArgs e)
        {
            if (czyWczytanyPlik) zapisz();
            else saveAsMenuItem_Click(sender, e);

        }

        private void closeMenuItem_Click(object sender, RoutedEventArgs e)
        {            
            Close();
        }

        private void textBox_TextChanged(object sender, System.Windows.Controls.TextChangedEventArgs e)
        {            
            czyTekstZmieniony = true;
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (czyTekstZmieniony)
            {
                MessageBoxResult wynik = MessageBox.Show("Czy chcesz zapisać zmiany" + (czyWczytanyPlik ? " w pliku " + ścieżkaPliku : "") + "?", Title, MessageBoxButton.YesNoCancel, MessageBoxImage.Question, MessageBoxResult.Cancel);
                switch(wynik)
                {
                    case MessageBoxResult.No:
                        break;
                    case MessageBoxResult.Yes:
                        saveMenuItem_Click(sender, new RoutedEventArgs());
                        if (!(bool)saveFileDialog.Tag) e.Cancel = true;
                        //KONKURS: eleganckie rozwiązanie problemu anulowania zapisu bez pól
                        break;
                    case MessageBoxResult.Cancel:
                        e.Cancel = true;
                        break;
                }
            }
        }

        private void newMenuItem_Click(object sender, RoutedEventArgs e)
        {
            if (czyTekstZmieniony)
            {
                MessageBoxResult wynik = MessageBox.Show("Czy jesteś pewien?", Title, MessageBoxButton.YesNo, MessageBoxImage.Question, MessageBoxResult.No);
                if(wynik == MessageBoxResult.Yes)
                {
                    textBox.Text = "";
                    czyTekstZmieniony = false;
                }
            }
        }

        private void cofnijMenuItem_Click(object sender, RoutedEventArgs e)
        {
            textBox.Undo();
        }

        private void powtórzMenuItem_Click(object sender, RoutedEventArgs e)
        {
            textBox.Redo();
        }

        private void wytnijMenuItem_Click(object sender, RoutedEventArgs e)
        {
            textBox.Cut();
        }

        private void kopiujMenuItem_Click(object sender, RoutedEventArgs e)
        {
            textBox.Copy();            
        }

        private void wklejMenuItem_Click(object sender, RoutedEventArgs e)
        {
            textBox.Paste();
        }

        private void usuńMenuItem_Click(object sender, RoutedEventArgs e)
        {
            textBox.SelectedText = "";
        }

        private void zaznaczWszystkoMenuItem_Click(object sender, RoutedEventArgs e)
        {
            textBox.SelectAll();
        }

        private void dataGodzinaMenuItem_Click(object sender, RoutedEventArgs e)
        {
            textBox.SelectedText = DateTime.Now.ToString();
            //textBox.SelectedText = DateTime.Now.ToString(new System.Globalization.CultureInfo("pl"));
            //textBox.SelectedText = DateTime.Now.ToString(System.Globalization.CultureInfo.InvariantCulture);
        }

        private void Window_PreviewKeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {            
            switch(e.Key)
            {
                case Key.F5: dataGodzinaMenuItem_Click(sender, null); e.Handled = true; break;
            }

            if ((e.KeyboardDevice.Modifiers & ModifierKeys.Control) == ModifierKeys.Control)
            {
                switch (e.Key)
                {
                    case Key.N: newMenuItem_Click(sender, null); e.Handled = true; break;
                    case Key.O: openMenuItem_Click(sender, null); e.Handled = true; break;
                    case Key.S: saveMenuItem_Click(sender, null); e.Handled = true; break;
                }
            }
        }

        private void zawijanieWierszyMenuItem_Click(object sender, RoutedEventArgs e)
        {
            MenuItem menuItem = sender as MenuItem;
            bool czyZaznaczone = menuItem.IsChecked;
            textBox.TextWrapping = czyZaznaczone ? TextWrapping.Wrap : TextWrapping.NoWrap;
        }

        private void pasekStanuMenuItem_Click(object sender, RoutedEventArgs e)
        {
            MenuItem menuItem = sender as MenuItem;
            bool czyZaznaczone = menuItem.IsChecked;
            pasekStanu.Visibility = czyZaznaczone ? Visibility.Visible : Visibility.Collapsed;            
        }

        private void drukujMenuItem_Click(object sender, RoutedEventArgs e)
        {
            PrintHelper.PrintText(textBox.Text);
        }
    }
}
